% This subroutine calculates the derivates of r(residue) with respect to Q
% (eigenvector matrix) and log of eigenvalues.

% J7 differs from J in that J7 doesn't return the derivatives
% of r with respect to the 7th terms of each eigenvector. Therefore, J7 is
% only used to calculate the error while assuming that the errors of the 7th
% terms of each eigenvector are zero.

function J=calculate_J7(data_d,beta)
    [~,m]=size(data_d); %number of experiments included
    Q = reshape(beta(1:49),7,7); % matrix of eigenvector
    num = 0; 
    n_start = 0;
    n_end = 0;
    JT=[];
    
    for i=1:m
        w1 = data_d(i).boundary(2:8,1);
        w2 = data_d(i).boundary(2:8,2);
        %for DC, w1 = w_left, w2 = w_right
        %for MD, w1 = w_interface, w2 = w_initial
        
        x0 = data_d(i).dist;
        t = data_d(i).time;
        error1 = data_d(i).error(2:8);
        error2 = data_d(i).error(1);
        
        if data_d(i).temp == 1260
            num = 0;
            Beta = beta(50:56);
        elseif data_d(i).temp == 1350
            num = 1;
            Beta = beta(57:63);
        else
            num = 2;
            Beta = beta(64:70);
        end
        
        n_start = num*7+1;
        n_end = (num+1)*7;
        
        if data_d(i).type==1   %for diffusion couple
            
            for j=1:data_d(i).np
                x= data_d(i).x(j);
                Y= (x-x0)*exp(-Beta/2)/(sqrt(4*t));
                E = diag( erf(Y) );
                dE = diag( -(x-x0)/sqrt(4*pi*t) * exp(-Y.*Y).*exp(-Beta/2) );
                wp = kron( (E *Q^-1*(w2-w1)/2), eye(7))-...
                    kron((Q^-1*(w2-w1)/2),(Q*E*Q^-1)');
                % wp is the partial derivative of concentration of 
                % independent components with respect to Q
                
                wBeta =diag(reshape(dE,49,1))...
                    *kron((Q^-1*(w2-w1)/2),Q');
                % wBeta is the partial derivative of concentration of
                % independent components with respect to Beta
            
                rp1 = -wp/diag(error1);
                % partial derivative of residue error of independent
                % components with respect with to Q
                
                rBeta1= -wBeta/diag(error1);
                % partial derivative of residue error of independent
                % components with respect with to Beta
                
                rp2 = sum(wp,2)/error2;
                % partial derivative of residue error of the dependent
                % components with respect with to Q
                
                rBeta2 = sum(wBeta,2)/error2;
                % partial derivative of residue error of the dependent
                % components with respect with to Beta 
                
                rp = [rp2,rp1];
                % partial derivative of residue error of all
                % components with respect with to Q
            
                rBeta3=[rBeta2,rBeta1];%49*8
                % partial derivative of residue error of all
                % components with respect with to Beta
                
                rBeta=zeros(21,8);
                for k=1:8
                    rBeta(n_start:n_end,k)= diag(reshape(rBeta3(:,k),7,7));
                end
                JT =[JT,[rp;rBeta]];
            end
            
        elseif data_d(i).type==2  % for mineral dissolution
            
            for j=1:data_d(i).np
                x= data_d(i).x(j);
                Y= (x-x0)*exp(-Beta/2)/(sqrt(4*t));
                Y0 = -x0*exp(-Beta/2)/(sqrt(4*t));
                
                E = erfc(Y);
                E0 = erfc(Y0);
                E_ratio = diag (E./E0);
                
                dE = (x-x0)/(sqrt(4*pi*t))*exp(-Y.*Y).*exp(-Beta/2);
                dE0 = -x0/(sqrt(4*pi*t))*exp(-Y0.*Y0).*exp(-Beta/2);
                
                dE_ratio = diag( ( dE.*E0 - dE0.*E )./(E0.^2) );
                
                wp = kron( (E_ratio*Q^-1*(w1-w2)), eye(7))-...
                    kron((Q^-1*(w1-w2)),(Q*E_ratio*Q^-1)'); 
                % wp is the partial derivative of concentration of 
                % independent components with respect to Q
                
                wBeta = diag(reshape(dE_ratio,49,1))...
                    *kron((Q^-1*(w1-w2)),Q');
                % wBeta is the partial derivative of concentration of
                % independent components with respect to Beta 
            
                rp1 = -wp/diag(error1); 
                % partial derivative of residue error of independent
                % components with respect with to Q
                
                rBeta1= -wBeta/diag(error1);
                % partial derivative of residue error of independent
                % components with respect with to Beta 
                
                
                rp2 = sum(wp,2)/error2;
                % partial derivative of residue error of the dependent
                % components with respect with to Q
                
                rBeta2 = sum(wBeta,2)/error2;
                % partial derivative of residue error of the dependent
                % components with respect with to Beta 
                
                rp = [rp2,rp1];
                % partial derivative of residue error of all
                % components with respect with to Q
                
                rBeta3=[rBeta2,rBeta1];%49*8
                % partial derivative of residue error of all
                % components with respect with to Beta 
                
                rBeta=zeros(21,8);
                for k=1:8
                    rBeta(n_start:n_end,k)= diag(reshape(rBeta3(:,k),7,7));
                end
                JT =[JT,[rp;rBeta]];
            end
        end
    end
    J1 = JT';
    
    [n,~]=size(J1);
    J=zeros(n,63);
    for i=0:6
        J(:,6*i+1:6*i+6)=J1(:,7*i+1:7*i+6);
    end
    J(:,43:63)=J1(:,50:70);
end